package net.cave.sortHolic;

/*
 * Software Design 2009 / 03
 * sample application
 * SortHolic
 * Author : Makoto Anjo
 * 
 */


import java.util.ArrayList;
import java.util.Random;
import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.AbsoluteLayout;
import android.widget.TextView;
import android.widget.AbsoluteLayout.LayoutParams;
import android.widget.Button;

public class SortHolic extends Activity {

	// マッチしてるかを表示する部分のサイズと座標指定
	final int MATCHING_X = 50;
	final int MATCHING_Y = 70;
	final int MATCHING_XSIZE = 24;
	final int MATCHING_YSIZE = 32;
	final int MATCHING_CHARS = 20;

	// 再挑戦(retry)ボタンのサイズと座標指定
	final int BUTTON_X = 380;
	final int BUTTON_Y = 10;
	final int BUTTON_XSIZE = 100;
	final int BUTTON_YSIZE = 50;
	
	final int DIGITS = 10; 	// 数字の数
	final int TEXT_SIZE = 36;
	final int GAP = 48;		// 表示間隔
	final int NUMBER_Y = 150;
	final int RND_MAX = 99;	// 乱数の上限
	final int PUSHED = 20;
	final int KIND_OF_SORT = 2;	// ソートの種類
	final int UNCHOSEN = -1;

	NumberText numbers[];
	TextView matching;
	int numberArr[];
	int chosenIndex;
	int step;
	boolean sortCombo[];
	ArrayList<Sort> sortUnit;
	QuickSort qs;
	BubbleSort bs;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	final AbsoluteLayout al;
        super.onCreate(savedInstanceState);
    	al = new AbsoluteLayout(this);
        setContentView(al);
        chosenIndex = UNCHOSEN;
        step = 0;
        
        matching = new TextView(this);
        matching.setTextSize(MATCHING_XSIZE);
        matching.setLayoutParams(new LayoutParams(
        		MATCHING_XSIZE*MATCHING_CHARS, MATCHING_YSIZE, MATCHING_X, MATCHING_Y));
        al.addView(matching);
        
    	sortUnit = new ArrayList<Sort>();
    	sortCombo = new boolean[KIND_OF_SORT];
        for(int i=0; i<KIND_OF_SORT; i++) {
        	sortCombo[i] = true;
        }
        
    	Random rnd = new Random();
        numbers = new NumberText[DIGITS];
        numberArr = new int[DIGITS];

		final Button retry = new Button(this);
		retry.setText("retry");
        retry.setLayoutParams(new LayoutParams(
        		BUTTON_XSIZE, BUTTON_YSIZE, BUTTON_X, BUTTON_Y));
		retry.setOnClickListener(new Button.OnClickListener() {
			public void onClick(View v){
				reset();
			}	
		});
        al.addView(retry);
        
        
        for(int i=0; i<DIGITS; i++) {
        	numbers[i] = new NumberText(this);
        	numbers[i].index = i;
    	    numbers[i].setOnClickListener(new View.OnClickListener() {
            	public void onClick(View v) {
            		if(chosenIndex == UNCHOSEN) { // 選択されてない場合
            			chosenIndex = ((NumberText)v).index;
            			moveWidget(v, ((NumberText)v).x, ((NumberText)v).y+PUSHED);
            		}
            		else { // 選択されている場合
            			// vとchosenIndexとの交換処理
            			if(((NumberText)v).index != chosenIndex) {
                			swapNumber(((NumberText)v).index, chosenIndex);
                			step++;
            			}
            			moveWidget(numbers[chosenIndex], numbers[chosenIndex].x, numbers[chosenIndex].y);
            			chosenIndex = UNCHOSEN;
            			checkPathes();

            			// 進行状況表示
            			if( isEqual(numberArr, qs.getArr())){
            				// ソート完了
                			matching.setText("おめでとう！ ソート完了！");
            			}
            			else {
                			int matchNum = countMatch();
                			if(matchNum == 0){
                    			matching.setText("No match");
                			}
                			else {
                    			matching.setText(matchNum + " sort alg matching");
                			}
            			}
            		}
            	}
            });

        	numberArr[i] = rnd.nextInt(RND_MAX);
        	Log.d("number", String.valueOf(numberArr[i]));
        }
        
        qs = new QuickSort(numberArr);
        qs.sort();
        sortUnit.add(qs);
        bs = new BubbleSort(numberArr);
        bs.sort();
        sortUnit.add(bs);
        
        
        for(int i=0; i<DIGITS; i++) {
            numbers[i].setText(twoDigit(numberArr[i]));
            numbers[i].setTextSize(TEXT_SIZE);
            numbers[i].setLayoutParams(new LayoutParams(
            		TEXT_SIZE*2, TEXT_SIZE*2, i*GAP, NUMBER_Y));  // magic number 2 (各数字が二ケタなので2倍)
            numbers[i].x = i*GAP;
            numbers[i].y = NUMBER_Y;
            al.addView(numbers[i]);
        }
    }
    
    private void reset(){
        chosenIndex = UNCHOSEN;
        step = 0;
        
		matching.setText("");
        
    	sortUnit = new ArrayList<Sort>();
    	sortCombo = new boolean[KIND_OF_SORT];
        for(int i=0; i<KIND_OF_SORT; i++) {
        	sortCombo[i] = true;
        }
        
    	Random rnd = new Random();
        numberArr = new int[DIGITS];
        
        for(int i=0; i<DIGITS; i++) {
        	numbers[i].index = i;
        	numberArr[i] = rnd.nextInt(RND_MAX);
        	Log.d("number", String.valueOf(numberArr[i]));
        }
        
        qs = new QuickSort(numberArr);
        qs.sort();
        sortUnit.add(qs);
        bs = new BubbleSort(numberArr);
        bs.sort();
        sortUnit.add(bs);
        
        
        for(int i=0; i<DIGITS; i++) {
            numbers[i].setText(twoDigit(numberArr[i]));
			moveWidget(numbers[i], numbers[i].x, numbers[i].y);
        }
    }

    private void swapNumber(int indexA, int indexB) {
    	int tmp = numberArr[indexA];
    	numberArr[indexA] = numberArr[indexB];
    	numberArr[indexB] = tmp;
    	
    	numbers[indexA].setText(twoDigit(numberArr[indexA]));
    	numbers[indexB].setText(twoDigit(numberArr[indexB]));
    }

    private void checkPathes() {
    	for(int i=0; i<sortUnit.size(); i++){
    		sortCombo[i] = checkPath(sortUnit.get(i), i);
        	Log.d("sortCombo[" + i + "]", String.valueOf(sortCombo[i]));
    	}
    }
    
    private boolean checkPath(Sort sr, int sortNum) {
    	if(sortCombo[sortNum] == false) {
    		return false;
    	}
    	return isEqual(sr.getArr(step), numberArr);
    }
    private int countMatch() {
    	int tmp = 0;
    	for(int i=0; i<sortCombo.length; i++){
    		if(sortCombo[i] == true) {
    			tmp++;
    		}
    	}
    	return tmp;
    }
    
    private boolean isEqual(int[] a, int[] b) {
    	if(a.length != b.length){
    		return false;
    	}
    	for(int i=0; i<a.length; i++) {
    		if(a[i] != b[i]){
    			return false;
    		}
    	}
    	return true;
    }
        
    public void moveWidget(View v, int x, int y) {
    	v.layout(x, y, x+v.getWidth(), y+v.getHeight());
    }       

    public String twoDigit(int a) {
    	if(a < 10) {
    		return "0" + String.valueOf(a);
    	}
    	else return String.valueOf(a);
    }
    
}